@@ -5,7 +5,7 @@
 c.factor = function(...)
 {
     args <- list(...)
-    for (i in seq(along=args)) if (!is.factor(args[[i]])) args[[i]] = as.factor(args[[i]])
+    for (i in seq_along(args)) if (!is.factor(args[[i]])) args[[i]] = as.factor(args[[i]])
     # The first must be factor otherwise we wouldn't be inside c.factor, its checked anyway in the line above.
     newlevels = unique(unlist(lapply(args,levels),recursive=TRUE,use.names=TRUE))
     ind <- fastorder(list(newlevels))
