@@ -26,6 +26,8 @@ setkeyv = function(x, cols, verbose=getOption("datatable.verbose"))
     if (!length(cols)) {
         cols = colnames(x)   # All columns in the data.table, usually a few when used in this form
     } else {
+        # remove backticks form cols 
+        cols <- gsub("`", "", cols)
         miss = !(cols %in% colnames(x))
         if (any(miss)) stop("some columns are not in the data.table: " %+% cols[miss])
     }
@@ -135,31 +137,50 @@ SJ = function(...) {
 
 # TO DO: Use the CJ list() method for SJ (and inside as.data.table.list?, #2109) too to avoid alloc.col
 
-CJ = function(...)
+CJ = function(..., sorted = TRUE)
 {
     # Pass in a list of unique values, e.g. ids and dates
     # Cross Join will then produce a join table with the combination of all values (cross product).
     # The last vector is varied the quickest in the table, so dates should be last for roll for example
     l = list(...)
-    # for (i in seq(along=l)) if (storage.mode(l[[i]])=="double") mode(l[[i]])="integer"
-    if (length(l)>1) {
-        n = sapply(l,length)
+    # for (i in seq_along(l)) if (storage.mode(l[[i]])=="double") mode(l[[i]])="integer"
+
+	# using rep.int instead of rep speeds things up considerably (but attributes are dropped).
+	j <- lapply(l, class) # changed "vapply" to avoid errors with "ordered" "factor" input
+	if (length(l)==1L && sorted && !identical(is.unsorted(l[[1L]]),FALSE)) {
+		l[[1L]] <- sort.int(l[[1L]], na.last=FALSE, method="quick")
+	} else if (length(l) > 1L) {
+        n = vapply(l, length, 0L)
         nrow = prod(n)
-        x=c(rev(take(cumprod(rev(n)))),1L)
-        for (i in seq(along=x)) l[[i]] = rep(l[[i]],each=x[i],length=nrow)
-    }
-    setattr(l,"row.names",.set_row_names(length(l[[1]])))
-    setattr(l,"class",c("data.table","data.frame"))
-    vnames = names(l)
-    if (is.null(vnames)) vnames=rep("",length(l))
-    tt = vnames==""
-    if (any(tt)) {
+        x = c(rev(take(cumprod(rev(n)))), 1L)
+        for (i in seq_along(x)) {
+			y <- l[[i]]
+            if (sorted && !identical(is.unsorted(y),FALSE))  # any NAs will cause a sort, even if they are at the beginning (can live with that)
+				y <- sort.int(y, na.last=FALSE, method="quick") # no worries for ties because there are no row.names or attributes to worry about.
+			if (i == 1L) 
+				l[[i]] <- rep.int(y, times = rep.int(x[i], n[i]))   # i.e. rep(y, each=x[i])
+			else if (i == length(n))
+				l[[i]] <- rep.int(y, times = nrow/(x[i]*n[i]))
+			else
+				l[[i]] <- rep.int(rep.int(y, times = rep.int(x[i], 
+					           n[i])), times = nrow/(x[i]*n[i]))
+		   if (any(class(l[[i]]) != j[[i]]))
+			   setattr(l[[i]], 'class', j[[i]]) # reset "Date" class - rep.int coerces to integer
+		}
+	}
+    setattr(l, "row.names", .set_row_names(length(l[[1L]])))
+    setattr(l, "class", c("data.table", "data.frame"))
+
+    if (is.null(vnames <- names(l))) 
+        vnames = vector("character", length(l)) 
+    if (any(tt <- vnames == "")) {
         vnames[tt] = paste("V", which(tt), sep="")
-        setattr(l,"names",vnames)
+	    setattr(l, "names", vnames)
     }
-    settruelength(l,0L)
-    l=alloc.col(l)  # a tiny bit wasteful to over-allocate a fixed join table (column slots only), doing it anyway for consistency, and it's possible a user may wish to use SJ directly outside a join and would expect consistent over-allocation.
-    setkey(l)    # TO DO: if inputs are each !is.unsorted, then no need to setkey here, just setattr("sorted") to save sort
+    settruelength(l, 0L)
+    l <- alloc.col(l)  # a tiny bit wasteful to over-allocate a fixed join table (column slots only), doing it anyway for consistency, and it's possible a user may wish to use SJ directly outside a join and would expect consistent over-allocation.
+    if (sorted) 
+		setattr(l, 'sorted', names(l))
     l
 }
 
