@@ -30,6 +30,8 @@ SEXP vecseq();
 SEXP copyattr();
 SEXP setlistelt();
 SEXP setnamed();
+SEXP address();
+SEXP copyNamedInList();
 
 // .Externals
 SEXP fastmean();
@@ -59,6 +61,8 @@ R_CallMethodDef callMethods[] = {
 {"Ccopyattr", (DL_FUNC) &copyattr, -1},
 {"Csetlistelt", (DL_FUNC) &setlistelt, -1},
 {"Csetnamed", (DL_FUNC) &setnamed, -1},
+{"Caddress", (DL_FUNC) &address, -1},
+{"CcopyNamedInList", (DL_FUNC) &copyNamedInList, -1},
 {NULL, NULL, 0}
 };
 
