@@ -4,7 +4,8 @@
 #include <Rdefines.h>
 //#include <sys/mman.h>
 #include <fcntl.h>
-extern int Rf_Scollate();   // #include <Defn.h> failed to find Defn.h in development, so this extern stops the warning
+
+int StrCmp(SEXP x, SEXP y);   // in countingcharacter.c
 
 /*
 Implements binary search (a.k.a. divide and conquer).
@@ -18,21 +19,31 @@ Differences over standard binary search (e.g. bsearch in stdlib.h) :
   o options to join to prevailing value (roll join a.k.a locf)
 */
 
-SEXP binarysearch(SEXP left, SEXP right, SEXP leftcols, SEXP rightcols, SEXP isorted, SEXP roll, SEXP rolltolast, SEXP nomatch, SEXP tolerance, SEXP retFirst, SEXP retLength, SEXP allLen1 )
+SEXP binarysearch(SEXP left, SEXP right, SEXP leftcols, SEXP rightcols, SEXP isorted, SEXP rollarg, SEXP rollends, SEXP nomatch, SEXP tolerance, SEXP retFirst, SEXP retLength, SEXP allLen1)
 {
     // If the left table is large and the right table is large, then sorting the left table first may be
     // quicker depending on how long to sort the left table. This is up to user via use of J() or SJ()
     
     R_len_t lr,nr,low,mid,upp,coln,col,lci,rci,len;
     R_len_t prevlow, prevupp, type, newlow, newupp, /*size,*/ lt, rt;
-    double tol = REAL(tolerance)[0];
-    SEXP lc, rc;
+    double tol = REAL(tolerance)[0], roll, rollabs;
+    Rboolean nearest=FALSE;
+    SEXP lc=NULL, rc=NULL;
+    if (isString(rollarg)) {
+        if (strcmp(CHAR(STRING_ELT(rollarg,0)),"nearest") != 0) error("roll is character but not 'nearest'");
+        roll=1.0; nearest=TRUE;       // the 1.0 here is just any non-0.0 
+    } else {
+        if (!isReal(rollarg)) error("Internal error: roll is not character or double");
+        roll = REAL(rollarg)[0];   // more common case (rolling forwards or backwards) or no roll when 0.0
+    }
+    rollabs = fabs(roll);
+    if (!isLogical(rollends) || LENGTH(rollends)!=2) error("rollends not a length 2 logical");
     union {
         int i;
         double d;
         SEXP s;
     } lval, rval;
-    if (NA_INTEGER > 0) error("expected internal value of NA_INTEGER %d to be negative",NA_INTEGER);
+    if (NA_INTEGER != INT_MIN) error("Internal error: NA_INTEGER (%d) != INT_MIN (%d).", NA_INTEGER, INT_MIN);
     if (NA_INTEGER != NA_LOGICAL) error("Have assumed NA_INTEGER == NA_LOGICAL (currently R_NaInt). If R changes this in future (seems unlikely), an extra case is required; a simple change.");
     nr = LENGTH(VECTOR_ELT(right,0));   // num rows in right hand table
     coln = LENGTH(leftcols);    // there may be more sorted columns in x than involved in the join
@@ -45,6 +56,7 @@ SEXP binarysearch(SEXP left, SEXP right, SEXP leftcols, SEXP rightcols, SEXP iso
         rt = TYPEOF(VECTOR_ELT(right, rci));
         if (lt != rt) error("typeof x.%s (%s) != typeof i.%s (%s)", CHAR(STRING_ELT(getAttrib(right,R_NamesSymbol),rci)), type2char(rt), CHAR(STRING_ELT(getAttrib(left,R_NamesSymbol),lci)), type2char(lt));
     }
+    if (nearest && TYPEOF(VECTOR_ELT(left, INTEGER(leftcols)[coln-1]))==STRSXP) error("roll='nearest' can't be applied to a character column");
     low=-1;
     for (lr=0; lr < LENGTH(VECTOR_ELT(left,0)); lr++) {  // left row (i.e. from i). TO DO: change left/right to i/x
         upp = prevupp = nr;
@@ -66,9 +78,9 @@ SEXP binarysearch(SEXP left, SEXP right, SEXP leftcols, SEXP rightcols, SEXP iso
                 while(low < upp-1) {
                     mid = low+((upp-low)/2);
                     rval.i = INTEGER(rc)[mid];
-                    if (rval.i<lval.i) {    // TO DO:  more efficient 3-way branch in C ?
+                    if (rval.i<lval.i) {          // relies on NA_INTEGER == INT_MIN, tested above with error if not
                         low=mid;
-                    } else if (rval.i>lval.i) {
+                    } else if (rval.i>lval.i) {   // TO DO:  more efficient 3-way branch using C ?.
                         upp=mid;
                     } else { // rval.i == lval.i)
                         // branch mid to find start and end of this == series [multi-column binary search]
@@ -112,13 +124,9 @@ SEXP binarysearch(SEXP left, SEXP right, SEXP leftcols, SEXP rightcols, SEXP iso
                             if (rval.s == lval.s) newupp=mid; else low=mid;
                         }
                         break;
-                    } else if (Rf_Scollate(rval.s, lval.s)<0) {
-                    // } else if (strcmp(CHAR(rval.s), CHAR(lval.s))<0) {
-                    // TO DO: test strcmp; i.e., if restricting to ASCII speeds up (maybe ASCII falls through and is
-                    // fast anyway)
-                    // Using Rf_Scollate caters for all, as does countingcharacter:ssort2. If we speed up later,
-                    // that's better than slowing down later. Switch can be a column level check that all is ascii
-                    // (setkey can check and mark)
+                    } else if (StrCmp(rval.s, lval.s)<0) {
+                    // TO DO: Reinvestigate non-ASCII. Switch can be a column level check that all is ascii
+                    // (setkey can check and mark). Used to use Rf_Scollate but was removed from r-devel API.
                     // We're using the last line of scmp in sort.c since we already dealt with NA and == above
                         low=mid;
                     } else {
@@ -129,11 +137,11 @@ SEXP binarysearch(SEXP left, SEXP right, SEXP leftcols, SEXP rightcols, SEXP iso
             case REALSXP :
                 // same comments for INTSXP apply here
                 lval.d = REAL(lc)[lr];
-                if (lval.d==NA_REAL) goto nextlr;
+                if (ISNAN(lval.d)) goto nextlr;
                 while(low < upp-1) {
                     mid = low+((upp-low)/2);
                     rval.d = REAL(rc)[mid];
-                    if (rval.d<lval.d-tol) {
+                    if (rval.d<lval.d-tol || ISNAN(rval.d)) {
                         low=mid;
                     } else if (rval.d>lval.d+tol) {
                         upp=mid;
@@ -158,17 +166,57 @@ SEXP binarysearch(SEXP left, SEXP right, SEXP leftcols, SEXP rightcols, SEXP iso
                 error("Type '%s' not supported as key column", type2char(type));
             }
         }
-        if (low<upp-1) {
-            INTEGER(retFirst)[lr] = low+2;   // extra +1 for 1-based indexing at R level
+        if (low<upp-1) {                   // if value found low and upp surround it, unlike standard binary search where low falls on it
+            INTEGER(retFirst)[lr] = low+2; // extra +1 for 1-based indexing at R level
             len = upp-low-1;
             INTEGER(retLength)[lr] = len;
             if (len > 1) LOGICAL(allLen1)[0] = FALSE;
-        } else {
-            if (low>prevlow && col==coln && (LOGICAL(roll)[0] ||
-                                            (LOGICAL(rolltolast)[0] && upp<prevupp))) {
-                INTEGER(retFirst)[lr] = low+1;
-                INTEGER(retLength)[lr] = 1;
-                low -= 1; // for test 148
+        } else if (roll!=0.0 && col==coln && lc && rc && (low>prevlow || upp<prevupp)) {
+            // '&& lc && rc' is for test 133 (empty x).  Testing double roll!=0.0 is ok here, i.e. !(roll==FALSE). 
+            // runs once per i row (not each search test), so not hugely time critical
+            if (low != upp-1) error("Internal error. low != upp-1");
+            if (low<prevlow) error("Internal error. low<prevlow");
+            if (upp>prevupp) error("Internal error. upp>prevupp");
+            if (nearest) {   // value of roll ignored currently when nearest
+                if ( low>prevlow && upp<prevupp ) {
+                    if (  ( TYPEOF(lc)==REALSXP && REAL(lc)[lr]-REAL(rc)[low] <= REAL(rc)[upp]-REAL(lc)[lr] )
+                       || ( TYPEOF(lc)<=INTSXP && INTEGER(lc)[lr]-INTEGER(rc)[low] <= INTEGER(rc)[upp]-INTEGER(lc)[lr] )) {
+                        INTEGER(retFirst)[lr] = low+1;
+                        INTEGER(retLength)[lr] = 1;
+                        low -= 1;
+                    } else {
+                        INTEGER(retFirst)[lr] = upp+1;
+                        INTEGER(retLength)[lr] = 1;
+                        upp += 1;
+                    }
+                } else if (upp==prevupp && LOGICAL(rollends)[1]) {
+                    INTEGER(retFirst)[lr] = low+1;
+                    INTEGER(retLength)[lr] = 1;
+                    low -= 1;
+                } else if (low==prevlow && LOGICAL(rollends)[0]) {
+                    INTEGER(retFirst)[lr] = upp+1;
+                    INTEGER(retLength)[lr] = 1;
+                    upp += 1;
+                }
+            } else {
+                if ( (   (roll>0.0 && low>prevlow && (upp<prevupp || LOGICAL(rollends)[1]))
+                      || (roll<0.0 && upp==prevupp && LOGICAL(rollends)[1]) )
+                  && (   (TYPEOF(lc)==REALSXP && REAL(lc)[lr]-REAL(rc)[low]-rollabs<tol)
+                      || (TYPEOF(lc)<=INTSXP && (double)(INTEGER(lc)[lr]-INTEGER(rc)[low])-rollabs<tol ) 
+                      || (TYPEOF(lc)==STRSXP)   )) {
+                    INTEGER(retFirst)[lr] = low+1;
+                    INTEGER(retLength)[lr] = 1;
+                    low -= 1; // for test 148
+                } else if 
+                   (  (  (roll<0.0 && upp<prevupp && (low>prevlow || LOGICAL(rollends)[0]))
+                      || (roll>0.0 && low==prevlow && LOGICAL(rollends)[0]) )
+                  && (   (TYPEOF(lc)==REALSXP && REAL(rc)[upp]-REAL(lc)[lr]-rollabs<tol)
+                      || (TYPEOF(lc)<=INTSXP && (double)(INTEGER(rc)[upp]-INTEGER(lc)[lr])-rollabs<tol )
+                      || (TYPEOF(lc)==STRSXP)   )) {
+                    INTEGER(retFirst)[lr] = upp+1;   // == low+2
+                    INTEGER(retLength)[lr] = 1;
+                    upp += 1;
+                }
             }
         }
         nextlr :;
